/*
 * Copyright (c) 2019 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: LicenseRef-Nordic-5-Clause
 */

/** @file
 *  @brief Nordic Battery Service Client sample
 */

#include <zephyr/types.h>

#include <errno.h>
#include <sys/printk.h>

#include <bluetooth/bluetooth.h>
#include <bluetooth/gatt.h>
//#include <bluetooth/gatt_dm.h>
#include <bluetooth/uuid.h>

#include <settings/settings.h>

#include <logging/log.h>


#define BT_GAP_ADV_INT_MIN        0x0020  /* 20 ms */
#define BT_GAP_ADV_INT_MAX        0x0020  /* 20 ms */

#define BT_ADV_5S_TIMEOUT         0x01F4  /* N=500 ;  Time = N * 10ms = 5s */
#define BT_ADV_30S_TIMEOUT        0x0BB8  /* N=3000 ; Time = N * 10ms = 30s */
#define BT_ADV_60S_TIMEOUT        0x1770  /* N=6000 ; Time = N * 10ms = 60s */


static uint8_t manu_data[7] = {   0xff, 0x3a,   /* Manufacturer */
                                  0x00, 0x00,   /* Major */
                                  0x00, 0x00,   /* Minor */
                                  0x00};        /* Nb of hops */

static const struct bt_data directed_b2b_ad[] = {
	BT_DATA_BYTES(BT_DATA_FLAGS, (BT_LE_AD_LIMITED | BT_LE_AD_NO_BREDR)),
        BT_DATA_BYTES(BT_DATA_UUID16_ALL, BT_UUID_16_ENCODE(0xFFFE)),
        BT_DATA(BT_DATA_MANUFACTURER_DATA, &manu_data, sizeof(manu_data)),
};

static bt_addr_le_t connecting_peer_addr;

static struct bt_le_adv_param directed_b2b_param =
                                      BT_LE_ADV_PARAM_INIT( BT_LE_ADV_OPT_CONNECTABLE | BT_LE_ADV_OPT_ONE_TIME | BT_LE_ADV_OPT_USE_NAME |
                                                            BT_LE_ADV_OPT_EXT_ADV | BT_LE_ADV_OPT_DIR_MODE_LOW_DUTY,
                                      BT_GAP_ADV_FAST_INT_MIN_2, \
                                      BT_GAP_ADV_FAST_INT_MAX_2, &connecting_peer_addr);


static struct bt_le_ext_adv *directed_b2b_adv;

//==============================================================================
// advertising: directed
//==============================================================================

static void directed_b2b_adv_sent(struct bt_le_ext_adv *adv,
                     struct bt_le_ext_adv_sent_info *info)
{
  printk("Directed B2B adv timeout\n");
}

static void directed_b2b_adv_connected(struct bt_le_ext_adv *adv,
			  struct bt_le_ext_adv_connected_info *info)
{
  printk("Directed B2B Connected\n");
}

static struct bt_le_ext_adv_cb directed_b2b_adv_callbacks =
{
  .sent = directed_b2b_adv_sent,
  .connected = directed_b2b_adv_connected,
  .scanned = NULL
};
      
//==============================================================================
// bt_ready bluetooth init
//==============================================================================

static void bt_ready(int err)
{
  // TODO: Handle  init errors
  if (err) {
    printk("Bluetooth init failed (err %d)\n", err);
    return;
  }

  printk("Bluetooth initialized\n");

  if (IS_ENABLED(CONFIG_SETTINGS)) {
          settings_load();
  }

  // Directed B2B Advetising Create
  err = bt_le_ext_adv_create(&directed_b2b_param, &directed_b2b_adv_callbacks, &directed_b2b_adv);
  if (err)
  {
    printk("Failed to create directed advertiser set (%d)", err);
    return;
  }

  // set extended directed B2B advertising data
  err = bt_le_ext_adv_set_data(directed_b2b_adv, directed_b2b_ad, ARRAY_SIZE(directed_b2b_ad), NULL, 0);
  if (err)
  {
    printk("Failed to set directed advertising data (%d)", err);
    return;
  }
  
  struct bt_le_ext_adv_start_param directed_b2b_adv_start_param = {
                                    .timeout = BT_ADV_30S_TIMEOUT, 
                                    .num_events = 0};
   
  connecting_peer_addr.type = 0x01;
  connecting_peer_addr.a.val[0] = 0xf0;
  connecting_peer_addr.a.val[1] = 0x0f;
  connecting_peer_addr.a.val[2] = 0x34;
  connecting_peer_addr.a.val[3] = 0x56;
  connecting_peer_addr.a.val[4] = 0x45;
  connecting_peer_addr.a.val[5] = 0xc7;

  directed_b2b_param.peer = &connecting_peer_addr;
  err = bt_le_ext_adv_update_param(directed_b2b_adv, &directed_b2b_param);
  if (err)
  {
    printk("Failed to update directed advertising data (%d)", err);
    return;
  }
  
  err = bt_le_ext_adv_start(directed_b2b_adv, &directed_b2b_adv_start_param);
  if (err)
  {
    printk("Failed to start Directed advertising(%d)\n", err);
    return;
  }
  
  char addr[BT_ADDR_LE_STR_LEN];
  bt_addr_le_to_str(directed_b2b_param.peer, addr, sizeof(addr));
  printk("Directed B2B Advertising Started addr: %s\n", addr);

}

//==============================================================================
// main
//==============================================================================

void main(void)
{
  int err;

  printk("11-directed_adv init\n");

  err = bt_enable(bt_ready);
  if (err) {
          printk("Bluetooth init failed (err %d)\n", err);
          return;
  }
}
